/*
 * Manual GPIO-based I2C communication.
 *
 * Copyright (C) 2013, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/gpio.h>
#include <stdint.h>

/* I2C modifiers. */

#define I2C_READ                    1
#define I2C_WRITE                   0

#ifdef __cplusplus

class I2c_gpio
{
  Hw::Gpio_chip *_scl_chip;
  int I2C_SCL;
  Hw::Gpio_chip *_sda_chip;
  int I2C_SDA;

  void CLR(int pin);
  void SET(int pin);
  void IN(int pin);
  int PIN(int pin);
  void wait();

public:
  explicit I2c_gpio(Hw::Gpio_chip *scl_chip, int scl, Hw::Gpio_chip *sda_chip, int sda);

  void start();
  void stop();
  void ack(bool ack);
  uint8_t recv();
  void recvmany(uint8_t *data, uint8_t len);
  bool send(uint8_t data);
  bool sendmany(uint8_t *data, uint8_t len);
};

#endif /* __cplusplus */



/* C language interface. */

EXTERN_C_BEGIN

void *i2c_gpio_get_channel(void *scl_chip, int scl, void *sda_chip, int sda);
void i2c_gpio_start(void *channel);
void i2c_gpio_stop(void *channel);
void i2c_gpio_ack(void *channel, bool ack);
uint8_t i2c_gpio_recv(void *channel);
void i2c_gpio_recvmany(void *channel, uint8_t *data, uint8_t len);
bool i2c_gpio_send(void *channel, uint8_t data);
bool i2c_gpio_sendmany(void *channel, uint8_t *data, uint8_t len);

EXTERN_C_END
